/*
 ============================================================================
 Name		 : ConsoleKeys.h
 Copyright   : Symbian Academy, 2007
 Description : Waits asynchronously for the next key press.
 ============================================================================
 */

#ifndef CONSOLEKEYS_H
#define CONSOLEKEYS_H

#include <e32base.h>	// For CActive, link against: euser.lib
#include <e32cons.h>    // Console
#include "Engine.h"

class CConsoleKeys : public CActive
	{
public:
	/**
	 * Two-phased constructor.
	 * @param aConsole reference to the console, the asynchronous service
	 * provider for this active object.
	 * @param aEngine engine class for callbacks when a key event has been
	 * received.
	 */
	static CConsoleKeys* NewL(CConsoleBase* aConsole, CEngine* aEngine);

	/** 
	 * Two-phased constructor - leaves the class on the cleanup stack.
	 * @param aConsole reference to the console, the asynchronous service
	 * provider for this active object.
	 * @param aEngine engine class for callbacks when a key event has been
	 * received.
	 */
	static CConsoleKeys* NewLC(CConsoleBase* aConsole, CEngine* aEngine);

	/** Cancel and destroy */
	~CConsoleKeys();

private:
	/**
	 * C++ constructor.
	 * Does not take ownership of the two parameters.
	 * @param aConsole reference to the console, the asynchronous service
	 * provider for this active object.
	 * @param aEngine engine class for callbacks when a key event has been
	 * received.
	 * */
	CConsoleKeys(CConsoleBase* aConsole, CEngine* aEngine);

	/**
	 * Second-phase constructor.
	 */
	void ConstructL();

public:
	/**
	 * Start waiting for key presses.
	 */
	void StartL();

private:
	/** 
	 * Handle completion of the asynchronous request.
	 * Call the KeyPressed() method of the engine.
	 * From CActive().
	 */
	void RunL();

	/**
	 * This implementation does the actual work of cancelling the active object.
	 * Note: Do not call this directly - instead, call Cancel() of the
	 * CActive base class, which will then call DoCancel().
	 */
	void DoCancel();

	/**
	 * Override RunError to handle leaves from RunL().
	 * Note: Not necessary for this exercise as RunL() does not cause a leave.
	 * @param aError error code that was thrown in RunL().
	 */
	//TInt RunError( TInt aError );

private:
	/** State of the active object */
	TInt iState;
	/** Reference to the console instance, the asynchronous service provider. */
	CConsoleBase* iConsole;
	/** Reference to the engine to notify it in case of a key press. */
	CEngine* iEngine;
	};

#endif // CONSOLEKEYS_H

